# Generated by Django 4.0.5 on 2022-06-03 13:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dynfilters', '0002_alter_dynamicfilterexpr_model'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dynamicfilterexpr',
            name='name',
            field=models.CharField(default='Report', max_length=128),
        ),
    ]
